import six


def encode_list(list_, encoding):
    if encoding:
        return [s.encode(encoding) for s in list_]
    return list_


def decode_list(list_, encoding):
    if encoding:
        return [s.decode(encoding) for s in list_]
    return list_


def decode_thing(thing, encoding):
    if not encoding:
        return thing
    if isinstance(thing, six.binary_type):
        return thing.decode(encoding)
    if hasattr(thing, '__iter__'):
        return decode_list(thing, encoding)
    # Punt
    return thing.decode(encoding)


def encode_thing(thing, encoding):
    if not encoding:
        return thing
    if isinstance(thing, six.text_type):
        return thing.encode(encoding)
    if hasattr(thing, '__iter__'):
        return encode_list(thing, encoding)
    return thing.encode(encoding)


def change_encoding(item, from_, to):
    if from_ == to:
        return item
    return encode_thing(decode_thing(item, from_), to)


class RecodingReader(six.Iterator):
    def __init__(self, stream, from_, to):
        self.stream = stream
        self.from_ = from_
        self.to = to

    def __next__(self):
        return change_encoding(six.next(self.stream), self.from_, self.to)

    def __iter__(self):
        return self



# -*- encoding: utf8 -*-
# Some code used in this module was taken from the python documentation and
# can be found in http://docs.python.org/2/library/csv.html
# Modified by Caktus.
from __future__ import unicode_literals, absolute_import


__all__ = ["DictReader", "DictWriter"]


from csv import DictReader as BaseDictReader, DictWriter as BaseDictWriter


class DictReader(BaseDictReader):
    def __init__(self, f, fieldnames=None, restkey=None, restval=None,
                 dialect="excel",
                 encoding=None,
                 *args, **kwds):
        BaseDictReader.__init__(self, f=f, fieldnames=fieldnames,
                                restkey=restkey, restval=restval,
                                dialect=dialect,
                                *args, **kwds)
        from .csv import reader
        self.reader = reader(f, dialect=dialect,
                             encoding=encoding,
                             **kwds)


class DictWriter(BaseDictWriter):
    def __init__(self, f, fieldnames, restval="", extrasaction="raise",
                 dialect="excel",
                 encoding=None,
                 *args, **kwds):
        from .csv import Writer

        BaseDictWriter.__init__(self, f=f, fieldnames=fieldnames,
                                restval=restval, extrasaction=extrasaction,
                                dialect=dialect, *args, **kwds)
        self.writer = Writer(csvfile=f, dialect=dialect, encoding=encoding,
                             **kwds)
        self.encoding = encoding



# -*- encoding: utf8 -*-
# Test the basic functions in the unicode-handling samples
# in the Python doc for CSV

# (Does not try to test the underlying csv library; leave that
# to the Python developers.)
from __future__ import unicode_literals
import codecs
import os.path
from unittest import TestCase

import six

import cordwainer.csv as csv3


class DictsTest(TestCase):
    def setUp(self):
        this_dir = os.path.dirname(__file__)
        self.data_dir = os.path.join(this_dir, "test_data")

    def test_dictreader(self):
        fname = os.path.join(self.data_dir, "utf8.csv")
        with open(fname, "rb") as f_binary:
            f = codecs.getreader('utf-8')(f_binary)
            reader = csv3.DictReader(f)
            rows = [row for row in reader]
            self.assertEqual(["head1", "hüad2", "head3"], reader.fieldnames)
            obj1 = dict(zip(["head1", "hüad2", "head3"], ["4", "5", "6"]))
            self.assertEqual(obj1, rows[0])
            obj2 = dict(zip(["head1", "hüad2", "head3"],
                            ["a b", "c d", "الأولى"]))
            self.assertEqual(obj2, rows[1])

    def test_dictreader_encodings(self):
        for encoding in ['utf-8']:
            fname = os.path.join(self.data_dir, "utf8.csv")
            with open(fname, "rb") as f_binary:
                data = f_binary.read().decode('utf-8')
            encoded_data = data.encode(encoding)
            input = six.BytesIO(encoded_data)
            reader = csv3.DictReader(input, encoding=encoding)
            rows = [row for row in reader]
            self.assertEqual(["head1", "hüad2", "head3"], reader.fieldnames)
            obj1 = dict(zip(["head1", "hüad2", "head3"], ["4", "5", "6"]))
            self.assertEqual(obj1, rows[0])
            obj2 = dict(zip(["head1", "hüad2", "head3"],
                            ["a b", "c d", "الأولى"]))
            self.assertEqual(obj2, rows[1])

    def test_dictwriter(self):
        output = six.StringIO()
        fieldnames = ["head1", "hüad2", "head3"]
        w = csv3.DictWriter(output, fieldnames)
        obj1 = dict(zip(["head1", "hüad2", "head3"],
                        ["4", "5", "6"]))
        obj2 = dict(zip(["head1", "hüad2", "head3"],
                        ["a b", "c d", "الأولى"]))
        w.writeheader()
        w.writerow(obj1)
        w.writerow(obj2)
        result = output.getvalue()
        self.assertEqual("head1,hüad2,head3\r\n4,5,6\r\na b,c d,الأولى\r\n",
                         result)

    def test_dictwriter_encodings(self):
        for encoding in ['utf-8', 'cp720']:
            output = six.BytesIO()
            fieldnames = ["head1", "head2", "head3"]
            w = csv3.DictWriter(output,
                                fieldnames,
                                encoding=encoding)
            obj1 = dict(zip(fieldnames,
                            ["4", "5", "6"]))
            obj2 = dict(zip(fieldnames,
                            ["a b", "c d", "الأولى"]))
            w.writeheader()
            w.writerow(obj1)
            w.writerow(obj2)
            data = output.getvalue()
            result = data.decode(encoding)
            self.assertEqual("head1,head2,head3\r\n"
                             "4,5,6\r\n"
                             "a b,c d,الأولى\r\n",
                             result)



# -*- encoding: utf8 -*-
# Some code used in this module was taken from the python documentation and
# can be found in http://docs.python.org/2/library/csv.html
# Modified by Caktus.
from __future__ import unicode_literals, absolute_import
import codecs

import six
from .utils import encode_list, decode_list, change_encoding, RecodingReader


__all__ = [ "QUOTE_MINIMAL", "QUOTE_ALL", "QUOTE_NONNUMERIC", "QUOTE_NONE",
            "Error", "Dialect", "__doc__", "excel", "excel_tab",
            "field_size_limit", "reader", "writer",
            "register_dialect", "get_dialect", "list_dialects", "Sniffer",
            "unregister_dialect", "__version__", "DictReader", "DictWriter" ]

# This file provides Python 3 compatible versions of the main tools
# in the csv module, with some added features.

# Provide everything from csv that we don't override
from csv import *  # flake8: noqa

# We want access to the stuff in csv ourselves, without getting
# confused aobut the things we've overridden
import csv

# Dicts are in another file, but let them be imported from here
from .dicts import DictReader, DictWriter


if six.PY2:
    # The built-in csv module expects streams to provide utf-8 encoded
    # data, and writes it that way to streams too.
    INTERNAL_ENCODING = 'utf-8'

if six.PY3:
    # Streams in and out of the built-in csv module are characters.
    INTERNAL_ENCODING = None


def reader(csvfile, dialect=excel, encoding=None, **kwargs):
    """
    Like Python 3's `csv.reader`, plus:

    * Can pass a byte stream and an encoding, and Reader will take
      care of decoding as it reads.
    * All results are strings
    * Can be used in Python 2.

    :param string encoding: If specified, `csvfile` is a byte stream
    with the specified encoding. `Reader` will decode it to string
    before processing it.  If None, `csvfile` is a character stream and
    `Reader` will not try to decode it.
    """
    return Reader(csvfile, dialect, encoding, **kwargs)


class Reader(six.Iterator):
    def __init__(self, csvfile, dialect=excel, encoding=None, **kwargs):
        self.line_num = 0
        self.csv_reader = csv.reader(
            RecodingReader(csvfile, from_=encoding, to=INTERNAL_ENCODING),
            dialect=dialect,
            **kwargs
        )

    def __next__(self):
        """Return next row, always as characters"""
        self.line_num += 1
        row = six.next(self.csv_reader)
        # Might need to decode it.
        return decode_list(row, INTERNAL_ENCODING)

    def __iter__(self):
        return self


class Writer(object):
    def __init__(self, csvfile, dialect='excel', encoding=None, **fmtparams):
        self.stream = csvfile
        self.queue = six.BytesIO() if six.PY2 else six.StringIO()
        self.writer = csv.writer(self.queue, dialect, **fmtparams)
        self.encoding = encoding

    @property
    def dialect(self):
        return self.writer.dialect

    def get_queue_data(self):
        """Return whatever data is in the queue, and clear out the queue"""
        data = self.queue.getvalue()
        self.queue.truncate(0)
        # You would think truncating would reset the file position,
        # but NOOOOOO......
        self.queue.seek(0)
        return data

    def writerow(self, row):
        # Write data to internal writer
        self.writer.writerow(change_encoding(row, from_=None, to=INTERNAL_ENCODING))
        # Get what it sent out
        data = self.get_queue_data()
        # Change that encoding to what we want to send to the output stream and write it
        out_data = change_encoding(data, from_=INTERNAL_ENCODING, to=self.encoding)
        self.stream.write(out_data)


def writer(csvfile, dialect='excel', encoding=None, **fmtparams):
    return Writer(csvfile, dialect, encoding, **fmtparams)



import sys

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


with open('README.rst') as file:
    long_description = file.read()


# http://tox.readthedocs.org/en/latest/example/basic.html#integration-with-setuptools-distribute-test-commands
class Tox(TestCommand):
    user_options = [('tox-args=', 'a', "Arguments to pass to tox")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.tox_args = None

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import tox
        import shlex
        errno = tox.cmdline(args=shlex.split(self.tox_args))
        sys.exit(errno)


setup(
    name='cordwainer',
    version='0.0.2',
    packages=find_packages(),
    url='https://github.com/caktus/cordwainer',
    license='APL2',
    author='Dan Poirier',
    author_email='dpoirier@caktusgroup.com',
    description='A better CSV module',
    install_requires=[
        'six>=1.7',
    ],
    tests_require=[
        'tox>=1.7',
        # Other test requirements are installed in each venv,
        #  per `tox.ini` file.
    ],
    cmdclass={'test': Tox},
    long_description=long_description,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries',
    ],
)



# -*- encoding: utf8 -*-
# Test the basic functions in the unicode-handling samples
# in the Python doc for CSV

# (Does not try to test the underlying csv library; leave that
# to the Python developers.)
from __future__ import unicode_literals
import codecs
import os.path
from unittest import TestCase
import six

import cordwainer.csv as csv3


class BasicsTest(TestCase):
    def setUp(self):
        this_dir = os.path.dirname(__file__)
        self.data_dir = os.path.join(this_dir, "test_data")
        self.rows = [
            ["head1", "hüad2", "head3"],
            ["4", "5", "6"],
            ["a b", "c d", "الأولى"]
        ]

    def test_reader(self):
        fname = os.path.join(self.data_dir, "utf8.csv")
        with open(fname, "rb") as f_binary:
            f = codecs.getreader("utf-8")(f_binary)
            reader = csv3.reader(f)
            rows = [row for row in reader]
            self.assertEqual(self.rows, rows)

    def test_reader_utf8(self):
        fname = os.path.join(self.data_dir, "utf8.csv")
        with open(fname, "rb") as f_binary:
            reader = csv3.reader(f_binary, encoding='utf-8')
            rows = [row for row in reader]
            self.assertEqual(self.rows, rows)

    def test_writer(self):
        # Get the output
        output = six.StringIO()
        w = csv3.writer(output, quoting=csv3.QUOTE_MINIMAL)
        w.writerow(["head1", "hüad2", "head3"])
        w.writerow(["a b", "c d", "الأولى"])
        result = output.getvalue()
        self.assertEqual("head1,hüad2,head3\r\na b,c d,الأولى\r\n", result)

    def test_writer_dialect(self):
        output = six.StringIO()
        delimiter = b'x' if six.PY2 else 'x'
        w = csv3.writer(output, quoting=csv3.QUOTE_ALL, delimiter=delimiter)
        dialect = w.dialect
        self.assertEqual(csv3.QUOTE_ALL, dialect.quoting)
        self.assertEqual(delimiter, dialect.delimiter)
        w = csv3.writer(output, quoting=csv3.QUOTE_NONE)
        dialect = w.dialect
        self.assertEqual(csv3.QUOTE_NONE, dialect.quoting)

    def test_writer_encodings(self):
        for encoding in ['utf-8']:
            output = six.BytesIO()
            w = csv3.writer(output, encoding=encoding,
                            lineterminator='\n')
            for row in self.rows:
                w.writerow(row)
            result = output.getvalue()
            chars = result.decode(encoding)
            fname = os.path.join(self.data_dir, "utf8.csv")
            expected = open(fname, "rb").read().decode('utf-8')
            self.assertEqual(expected, chars)
